@@ -37,6 +37,8 @@ pub mod remote;
 pub mod repo;
 /// Release statistics.
 pub mod statistics;
+/// Changelog commit processing summary.
+pub mod summary;
 /// Git tag.
 pub mod tag;
 /// Template engine.
