@@ -149,6 +149,9 @@ pub struct Opt {
 		num_args = 0..=1,
 	)]
     pub with_tag_message: Option<String>,
+    /// Sets the tags to skip in the changelog.
+    #[arg(long, env = "GIT_CLIFF_SKIP_TAGS", value_name = "PATTERN")]
+    pub skip_tags: Option<Regex>,
     /// Sets the tags to ignore in the changelog.
     #[arg(long, env = "GIT_CLIFF_IGNORE_TAGS", value_name = "PATTERN")]
     pub ignore_tags: Option<Regex>,
@@ -359,6 +362,9 @@ pub struct Opt {
     /// Load TLS certificates from the native certificate store.
     #[arg(long, help_heading = Some("FLAGS"), hide = !cfg!(feature = "remote"))]
     pub use_native_tls: bool,
+    /// Disable network access for remote repositories.
+    #[arg(long, help_heading = Some("REMOTE OPTIONS"), hide = !cfg!(feature = "remote"))]
+    pub offline: bool,
 }
 
 /// Custom type for the remote value.
